function [segInt,pX,pY,interval] = lineSegInt(x1,y1,x2,y2,epts,showPlot)
%LINESEGINT fast implementation of intersection of two line segments
% lineSegInt determines the intersection point of two line segments defined
% by [x1 y1] and [x2 y2] where x1, x2, y1, y2 are each two element vectors.  
% Parallelism and the intersection point for non-parallel lines are  
% calculated using the method of determinants.  Provides a faster 
% alternative for finding intersections for the simple two line segment 
% case than using more flexible functions such as intersections.m or 
% curveintersect.m
%
% Output "segInt" is true if the segments intersect and the intersection 
% point(s) lies on both line segments, and false otherwise.
%
% Outputs "pX" and "pY" specify the the intersection of the lines defined by 
% the input line segments.  The intersection may lie off one or both line 
% segments.
% - If the segments are not parallel [pX pY] specifies the actual 
%   intersection point
% - If the segments are parallel but not colinear pX and pY return nan 
%   indicating that there is no intersection.
% - If the segments are colinear pX and pY return inf indicating an infinite 
%   number of intersection points (which may or may not lie on the input 
%   line segments)
%
% Output "interval" specifies the range of on-segment intercepts.
% - For non-parallel segments interval returns [pX pY] if the intercept 
%   occurs on both line segments, and returns [] otherwise.
% - For parallel, non-overlapping segments interval returns []
% - For parallel, overlapping segments interval returns[[x1; x2] [y1; y2]] 
%   where (x1,y1) and (x2,y2) are the endpoints of the overlapping segment 
%
% If optional argument "showPlot" is true or non-zero the line segments and 
% intersection points are plotted in a new figure to allow visualization 
% and debugging
%
% Examples:
%   Intersection of segments:
%     [segInt,pX,pY,interval] = lineSegInt([0 4],[0 4],[0 3],[1 2])
%     segInt = 
%          1
%     pX =
%         1.5000
%     pY =
%         1.5000
%     interval =
%         1.5000    1.5000
%
%   Intersection of extended lines that occurs off the segments:
%     [segInt,pX,pY,interval] = lineSegInt([0 4],[0 4],[0 3],[3 4])
%     segInt =
%          0
%     pX =
%         4.5000
%     pY =
%         4.5000
%     interval =
%          []
%   Parallel, non-colinear segments:
%     [segInt,pX,pY,interval] = lineSegInt([1 3],[3 1],[0 2],[3 1])
%     segInt =
%          0
%     pX =
%        NaN
%     pY =
%        NaN
%     interval =
%          []
%
%   Colinear, non-overlaping segments:
%     [segInt,pX,pY,interval] = lineSegInt([1 3],[0 0],[0 -2],[0 0])
%     segInt =
%          0
%     pX =
%        Inf
%     pY =
%        Inf
%     interval =
%          []
%
%   Colinear, overlapping segments
%     [segInt,pX,pY,interval] = lineSegInt([1 1],[1 -2],[1 1],[4 -3])
%     segInt =
%          1
%     pX =
%        Inf
%     pY =
%        Inf
%     interval =
%          1    -2
%          1     1

% Author: Joe Shaw
% jshaw3@gmu.edu

if nargin < 5
    epts = 0;
end
ept = abs(epts);

interval = [];

% Determine parallelism using determinants
dx1 = x1(1) - x1(2);
dy1 = y1(1) - y1(2);
dx2 = x2(1) - x2(2);
dy2 = y2(1) - y2(2);
denom = dx1*dy2 - dy1*dx2;

if denom ~= 0
    % Lines are not parallel -- compute intersection directly using determinants
    det1 = x1(1)*y1(2) - y1(1)*x1(2);
    det2 = x2(1)*y2(2) - y2(1)*x2(2);

    xN = det1*dx2 - dx1*det2;
    yN = det1*dy2 - dy1*det2;    
    
    pX = xN/denom;
    pY = yN/denom;
    
    x1s = sort(x1);
    y1s = sort(y1);
    x2s = sort(x2);
    y2s = sort(y2);
    
%     segInt = min(x1(:)) <= pX && pX <= max(x1(:)) && ...
%         min(x2(:)) <= pX && pX <= max(x2(:)) && ...
%         min(y1(:)) <= pY && pY <= max(y1(:)) && ...
%         min(y2(:)) <= pY && pY <= max(y2(:));
    % Intercept is on segment if it falls between endpoints on both segments
    % with some specified tolerance
    segInt =    x1s(1)-ept <= pX && pX <= x1s(2)+ept && ...
                x2s(1)-ept <= pX && pX <= x2s(2)+ept && ...
                y1s(1)-ept <= pY && pY <= y1s(2)+ept && ...
                y2s(1)-ept <= pY && pY <= y2s(2)+ept;
    
    % Consider intercept not on segment if it is arbitrarily close to any of the endpoints
    if segInt && epts > 0 
        if any(sqrt(([x1(:); x2(:)]-pX).^2 + ([y1(:); y2(:)]-pY).^2) < ept)
            segInt = 0;
        end
    end
    if segInt
        interval = [pX pY];
    end
        
else
    % Lines are parallel -- check special cases first
    if dx1 == 0
        % Vertical parallel lines
        if x1(1) ~= x2(1)
            % Not colinear
            pX = nan;
            pY = nan;
            segInt = 0;
        else
            % Lines are colinear
            pX = inf;
            pY = inf;
            yIntMin = max(min(y1(:)),min(y2(:)));
            yIntMax = min(max(y1(:)),max(y2(:)));            
            segInt = yIntMin <= yIntMax; 
            if segInt
                interval = [[x1(1); x1(2)] [yIntMin; yIntMax]];
            end
        end
            
    elseif dy1 == 0
        % Horizontal parallel lines
        if y1(1) ~= y2(1)
            % Not colinear
            pX = nan;
            pY = nan;
            segInt = 0;
        else
            % Lines are colinear
            pX = inf;
            pY = inf;            
            xIntMin = max(min(x1(:)),min(x2(:)));
            xIntMax = min(max(x1(:)),max(x2(:)));            
            segInt = xIntMin <= xIntMax;
            if segInt
                interval = [[xIntMin; xIntMax] [y1(1); y1(2)]];
            end
        end

    else
        % Parallel lines with general slope
        m = dy1/dx1;
        b1 = y1-m*x1;
        b2 = y2-m*x2;
        if b1 ~= b2
            % Not colinear
            pX = nan;
            pY = nan;
            segInt = 0;
        else
            % Lines are colinear
            pX = inf;
            pY = inf;
            xIntMin = max(min(x1(:)),min(x2(:)));
            xIntMax = min(max(x1(:)),max(x2(:)));
            segInt = xIntMin <= xIntMax;
            if segInt
                interval = [[xIntMin; xIntMax] [xIntMin*m+b1; xIntMax*m+b1]];
            end
        end        
    end
end

% Plot segments and intersection
if nargin == 6 && showPlot
    figure; hold on
    plot(x1,y1,'r')
    plot(x2,y2,'b')
    
    if segInt
        plot(interval(:,1),interval(:,2),'k-o')
        disp('Intercept occurs on segment')
    else 
        plot(pX,pY,'kx')
        disp('No segment intercept')
    end
end
